#!/usr/bin/perl

package Cognition::Export::BlogEntry;
use Cognition::Export;
use Cognition::Export::Contact;
use HTML::Entities qw(encode_entities_numeric);
use MIME::Base64;
use URI;

sub is
{
	my $subject = shift;
	return unless ($subject);
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://www.w3.org/2002/12/cal/ical#Vjournal')
			||  ($type eq 'urn:ietf:rfc:2445#vjournal')
			||  ($type eq 'urn:ietf:rfc:4287#entry')
			||  ($type eq 'http://purl.org/rss/1.0/item'));
	}
	return;
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		},
		{
			code  => 'Atom',
			name  => 'Atom Feed',
			type  => 'application/atom+xml'
		}
	];
}


sub _outAtom
{
	my $triples = $_[0];
	my $e       = $_[1];
	my $rv      = '';
	
	my $E = findSubject($triples, $e);
	
	# Todo: GeoRSS!
	
	$rv .= "\t<entry>\n";

	my $link = firstOf($E,
		'urn:ietf:rfc:4287#link',
		'http://purl.org/rss/1.0/link',
		'urn:ietf:rfc:2445#uid',
		'http://www.w3.org/2002/12/cal/ical#uid');
	$rv .= "\t\t<id>".encode_entities_numeric($link)."</id>\n";
	$rv .= "\t\t<link rel=\"self\" href=\"".encode_entities_numeric($link)."\" />\n";

	my $title = firstOf($E,
		'urn:ietf:rfc:4287#title',
		'http://purl.org/rss/1.0/title',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title',
		'urn:ietf:rfc:2445#summary',
		'http://www.w3.org/2002/12/cal/ical#summary');
	$title = 'Untitled' if (!$title);
	$rv .= "\t\t<title>".encode_entities_numeric($title)."</title>\n";

	my $content = firstOf($E,
		'urn:ietf:rfc:4287#content',
		'http://purl.org/rss/1.0/description',
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description',
		'urn:ietf:rfc:2445#description',
		'http://www.w3.org/2002/12/cal/ical#description');
	if ($content)
	{
		$rv .= "\t\t<content>".encode_entities_numeric($content)."</content>\n";
	}
	else
	{
		$rv .= "\t\t<link rel=\"alternate\" href=\"".encode_entities_numeric($link)."\" />\n";
	}
	
	# 'urn:ietf:rfc:2445#attach',
	# 'http://www.w3.org/2002/12/cal/ical#attach'
	my @encs = allOfObj($E, 'urn:ietf:rfc:4287#enclosure');
	foreach my $enc (@encs)
	{
		my $E = findSubject($triples, $enc->uri);
		if ($E)
		{
			my $euri = $E->firstVal('http://microformats.org/wiki/rel-enclosure#enclosure');
			if ($euri)
			{
				$rv .= "\t\t<link rel=\"enclosure\" href=\"".encode_entities_numeric($euri)."\" />\n";
			}
		}
		elsif ($enc->type)
		{
			$rv .= "\t\t<link rel=\"enclosure\" href=\"".encode_entities_numeric($enc->toString)."\" />\n";
		}
	}

	my $summary = firstOf($E,
		'urn:ietf:rfc:4287#summary',
		'http://purl.org/dc/terms/abstract',
		'urn:ietf:rfc:2445#comment',
		'http://www.w3.org/2002/12/cal/ical#comment');
	$rv .= "\t\t<summary>".encode_entities_numeric($summary)."</summary>\n"
		if ($summary);

	my @authors = allOfObj($E,
		'urn:ietf:rfc:4287#author',
		'http://purl.org/dc/terms/creator',
		'http://purl.org/dc/elements/1.1/creator',
		'http://xmlns.com/foaf/0.1/maker',
		'urn:ietf:rfc:2445#organizer',
		'http://www.w3.org/2002/12/cal/ical#organizer');
	foreach my $author (@authors)
	{
		my $authorSubject = findSubject($triples, $author->toString);
		if ($authorSubject && Cognition::Export::Contact::is($authorSubject))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$authorSubject->uri,
				"\t\t<author>");
		}
		elsif ($author->type)
		{
			$rv .= "\t\t<author><url>".$author->toString."</url></author>\n";
		}
		else
		{
			$rv .= "\t\t<author><name>".$author->toString."</name></author>\n";
		}
	}

	my @contributors = allOfObj($E,
		'urn:ietf:rfc:4287#contributor',
		'http://purl.org/dc/terms/contributor',
		'http://purl.org/dc/elements/1.1/contributor');
	foreach my $contributor (@contributors)
	{
		my $contributorSubject = findSubject($triples, $contributor->toString);
		if ($contributorSubject && Cognition::Export::Contact::is($contributorSubject))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$contributorSubject->uri,
				"\t\t<contributor>");
		}
		elsif ($contributor->type)
		{
			$rv .= "\t\t<contributor><url>".$contributor->toString."</url></contributor>\n";
		}
		else
		{
			$rv .= "\t\t<contributor><name>".$contributor->toString."</name></contributor>\n";
		}
	}

	my @cats = allOfObj($E, 'urn:ietf:rfc:4287#category',
		'urn:ietf:rfc:2445#categories',
		'http://www.w3.org/2002/12/cal/ical#categories');
	foreach my $cat (@cats)
	{
		my ($tag, $scheme, $label);
		my $catSubject = findSubject($triples, $cat->toString);
		
		if ($catSubject)
		{
			$tag    = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#tag',
				'http://purl.org/dc/terms/title',
				'http://purl.org/dc/elements/1.1/title');
			$scheme = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#tagspace');
			$label  = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#label');
		}
		$tag   = $cat->toString if (!$tag);
		$label = $cat->toString if (!$label);
		
		$rv .= "\t\t<category term=\"".encode_entities_numeric($tag)."\" ";
		$rv .= "scheme=\"".encode_entities_numeric($scheme)."\" "
			if (length $scheme);
		$rv .= "label=\"".encode_entities_numeric($label)."\" "
			if (length $label);
		$rv .= "/>\n";
	}

	my $rights = firstOf($E,
		'urn:ietf:rfc:4287#rights',
		'http://purl.org/dc/terms/rights',
		'http://purl.org/dc/elements/1.1/rights');
	$rv .= "\t\t<rights>".encode_entities_numeric($rights)."</rights>\n"
		if ($rights);

	my $published = firstOf($E,
		'urn:ietf:rfc:4287#published',
		'http://purl.org/dc/terms/available',
		'http://purl.org/dc/terms/created',
		'urn:ietf:rfc:2445#created',
		'http://www.w3.org/2002/12/cal/ical#created');
	$published =~ s/(\d\d)(\d\d)$/$1:$2/;
	$rv .= "\t\t<published>".encode_entities_numeric($published)."</published>\n"
		if ($published);

	my $updated = firstOf($E,
		'urn:ietf:rfc:4287#updated',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date',
		'urn:ietf:rfc:2445#dtstamp',
		'http://www.w3.org/2002/12/cal/ical#dtstamp',
		'urn:ietf:rfc:2445#last-modified',
		'http://www.w3.org/2002/12/cal/ical#last-modified');
	if (!$updated)
	{
		$updated = $published;
	}
	if ($updated)
	{
		$updated =~ s/(\d\d)(\d\d)$/$1:$2/;
		$rv .= "\t\t<updated>".encode_entities_numeric($updated)."</updated>\n";
		$_[2] = $updated
			if ((''.$_[2]) lt "$updated");
	}
	
	$rv .= "\t</entry>\n";

	return $rv;
}


sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $triples = shift;
	my $subject = shift;
	my $S = findSubject($triples, $subject);
	my @rv = (['BEGIN','VJOURNAL']);

	# UID
	my $uid = firstOf($S, 'urn:ietf:rfc:2445#uid',
		'http://www.w3.org/2002/12/cal/ical#uid',
		'http://purl.org/dc/terms/identifier',
		'http://purl.org/dc/elements/1.1/identifier',
		'urn:ietf:rfc:4287#link',
		'http://purl.org/rss/1.0/link');
	push @rv, ['UID', $uid]
		if ($uid);
	
	# Related-To;RelType=Child
	my @kids = allOf($S, 'urn:ietf:rfc:2445#child');
	foreach my $kid (@kids)
	{
		my $kidSubject = findSubject($triples, $kid);
		if ($kidSubject)
		{
			my $kUid = firstOf($kidSubject, 'urn:ietf:rfc:2445#uid',
				'http://www.w3.org/2002/12/cal/ical#uid',
				'http://purl.org/dc/terms/identifier',
				'http://purl.org/dc/elements/1.1/identifier');
			if ($kUid)
			{
				push @rv, ['RELATED-TO;RELTYPE=CHILD', $kUid]
			}
		}
	}

	# Related-To;RelType=Sibling
	my @sibs = allOf($S, 'urn:ietf:rfc:2445#sibling');
	foreach my $sib (@sibs)
	{
		my $sibSubject = findSubject($triples, $sib);
		if ($sibSubject)
		{
			my $sUid = firstOf($triples->{$sib}, 'urn:ietf:rfc:2445#uid',
				'http://www.w3.org/2002/12/cal/ical#uid',
				'http://purl.org/dc/terms/identifier',
				'http://purl.org/dc/elements/1.1/identifier');
			if ($sUid)
			{
				push @rv, ['RELATED-TO;RELTYPE=SIBLING', $sUid]
			}
		}
	}

	# Related-To;RelType=Parent
	my $parent = firstOf($S, 'urn:ietf:rfc:2445#parent');
	if ($parent)
	{
		my $parentSubject = findSubject($triples, $parent);
		if ($parentSubject)
		{
			my $pUid = firstOf($parentSubject, 'urn:ietf:rfc:2445#uid',
				'http://www.w3.org/2002/12/cal/ical#uid',
				'http://purl.org/dc/terms/identifier',
				'http://purl.org/dc/elements/1.1/identifier');
			if ($pUid)
			{
				push @rv, ['RELATED-TO;RELTYPE=PARENT', $pUid]
			}
		}
	}

	# Attach
	my @attachments = allOfObj($S, 'urn:ietf:rfc:2445#attach',
		'http://www.w3.org/2002/12/cal/ical#attach',
		'http://purl.org/dc/terms/hasPart',
		'urn:ietf:rfc:4287#enclosure');
	foreach my $att (@attachments)
	{
		my $Att = findSubject($triples, $att->toString);
		if ($Att)
		{
			my $e = $Att->firstVal('http://microformats.org/wiki/rel-enclosure#enclosure');
			push @rv, ['ATTACH;VALUE=uri',$e] if ($e);
		}
		elsif ($att->type)
		{
			push @rv, ['ATTACH;VALUE=uri',$att->toString];
		}
		else
		{
			push @rv, ['ATTACH',$att->toString];
		}
	}
	
	# Categories
	my @cats = allOfObj($S, 'urn:ietf:rfc:2445#categories',
		'http://www.w3.org/2002/12/cal/ical#categories',
		'http://purl.org/rss/1.0/modules/event/type');
	my $catstring = '';
	foreach my $cat (@cats)
	{
		my $catSubject = findSubject($triples, $cat->toString);
		if ($catSubject)
		{
			my $category = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#tag',
				'http://purl.org/dc/terms/description',
				'http://purl.org/dc/elements/1.1/description');
			$catstring .= ','.uc($category)
				if (length $category);
		}
		else
		{
			my $category = $cat->toString;
			$category =~ s/(^\s+|\s+$)//g;
			$category =~ s/\s+/ /g;
			$catstring .= ','.uc($category);
		}
	}
	$catstring =~ s/^\,//;
	push @rv, ['CATEGORIES',$catstring,1]
		if ($catstring);

	# Class
	my $class = firstOf($S, 'urn:ietf:rfc:2445#class',
		'http://www.w3.org/2002/12/cal/ical#class');
	push @rv, ['CLASS', uc($class)]
		if ($class);	

	# comment
	my $comm = firstOf($S, 'urn:ietf:rfc:2445#comment',
		'http://www.w3.org/2002/12/cal/ical#comment',
		'urn:ietf:rfc:4287#summary',
		'http://purl.org/dc/terms/abstract');
	push @rv, ['SUMMARY', $summary]
		if ($summary);

	# Geo
	# Todo: GeoRSS!
	my @geos = allOfObj($S, 'urn:ietf:rfc:2445#geo',
		'http://www.w3.org/2002/12/cal/ical#geo');
	foreach my $geo (@geos)
	{
		my $geoSubject = findSubject($triples, $geo->toString);
		if ($geoSubject)
		{
			if (Cognition::Export::Point::is($geoSubject))
			{
				my ($key, $val) = Cognition::Export::Point::outVCFLine($triples, $geoSubject->uri);
				push @rv, [$key, $val, 1];
			}
		}
	}

	# description
	my @desc = allOf($S, 'urn:ietf:rfc:4287#content',
		'http://purl.org/rss/1.0/description',
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description',
		'urn:ietf:rfc:2445#description',
		'http://www.w3.org/2002/12/cal/ical#description');
	foreach my $d (@desc)	
		{ push @rv, ['DESCRIPTION', $d]; }
	
	# Status
	my $status = firstOf($S, 'urn:ietf:rfc:2445#status',
		'http://www.w3.org/2002/12/cal/ical#status');
	push @rv, ['STATUS', uc($status)]
		if ($status);	

	# Summary
	my $summary = firstOf($S, 'urn:ietf:rfc:2445#summary',
		'http://www.w3.org/2002/12/cal/ical#summary',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title',
		'urn:ietf:rfc:4287#title',
		'http://purl.org/rss/1.0/title');
	push @rv, ['SUMMARY', $summary]
		if ($summary);

	# URL
	my $url = firstOf($S, 'urn:ietf:rfc:2445#url',
		'http://www.w3.org/2002/12/cal/ical#url');
	push @rv, ['URL', $url]
		if ($url);
		
	# Dates and stuff
	my $dt = firstOf($S, 'urn:ietf:rfc:2445#created',
		'http://www.w3.org/2002/12/cal/ical#created',
		'urn:ietf:rfc:4287#published',
		'http://purl.org/dc/terms/available',
		'http://purl.org/dc/terms/created');
	push @rv, ['CREATED', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#dtstamp',
		'http://www.w3.org/2002/12/cal/ical#dtstamp',
		'urn:ietf:rfc:4287#updated',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	push @rv, ['DTSTAMP', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#last-modified',
		'http://www.w3.org/2002/12/cal/ical#last-modified',
		'urn:ietf:rfc:4287#updated',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	push @rv, ['LAST-MODIFIED', $dt] if ($dt);

	# RDATE, RRULE, EXDATE, EXRULE - o'reilly?!
	foreach my $p (@{['rdate', 'rrule', 'exdate', 'exrule']})
	{
		my @R = allOf($S, 'urn:ietf:rfc:2445#'.$p,
			'http://www.w3.org/2002/12/cal/ical#'.$p);
		foreach my $r (@R)
			{ push @rv, [uc($p), $r, 1]; }
	}
	# Recurrence-ID
	my $rid = firstOf($S, 'urn:ietf:rfc:2445#recurrence-id',
		'http://www.w3.org/2002/12/cal/ical#recurrence-id');
	push @rv, ['RECURRENCE-ID', $rid]
		if ($rid);

	# Sequence
	my $seq = firstOf($S, 'urn:ietf:rfc:2445#sequence',
		'http://www.w3.org/2002/12/cal/ical#sequence');
	push @rv, ['SEQUENCE', $seq] if ($seq);
	
	# Description
	my $desc = firstOf($S, 'urn:ietf:rfc:2445#description',
		'http://www.w3.org/2002/12/cal/ical#description',
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description',
		'http://purl.org/dc/terms/abstract');
	push @rv, ['DESCRIPTION', $desc]
		if ($desc);
	
	# Attendees, Contact, Organizer
	push @rv, Cognition::Export::CalComponent::_people($triples, $S, 'aco');

	# Done.
	push @rv, ['END','VJOURNAL'];
	
	# Convert to text/directory syntax.
	my $rv = '';
	foreach my $line (@rv)
	{
		my ($k, $v, $soft) = @{$line};
		
		if ($k =~ /VALUE.uri$/i && $v =~ /^data\:/)
		{
			my $uri = URI->new($v);
			$v = encode_base64($uri->data, '');
			$k =~ s/VALUE.uri$/ENCODING\=b/i;
			$k .= ';TYPE='.$uri->media_type
				if ($uri->media_type);
		}
		
		$v =~ s/\\/\\\\/g;
		$v =~ s/\r?\n/\\n/g;
		unless ($soft)
		{
			$v =~ s/\;/\\\;/g;
			$v =~ s/\,/\\\,/g;
		}
		
		$rv .= "$k:$v\r\n";
	}
	return $rv;
}

1;
